package com.example.e13roomshoppinglist


import android.content.Context
import android.util.Log
import androidx.compose.animation.animateColorAsState
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.horizontalScroll
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.sizeIn
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.DismissDirection
import androidx.compose.material3.DismissState
import androidx.compose.material3.DismissValue
import androidx.compose.material3.Divider
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.RadioButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SwipeToDismiss
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.rememberDismissState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.scale
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.compose.ui.viewinterop.AndroidView
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.AdSize
import com.google.android.gms.ads.AdView
import kotlin.random.Random


@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun ShoppingListScreen(
    state: ShoppingListState, onEvent: (ShoppingListEvent) -> Unit, onShowToast: (String) -> Unit?
) {
    var shouldShowInterstitialAd by rememberSaveable {
        mutableStateOf(false)
    }
    var shouldShowRewardAd by rememberSaveable {
        mutableStateOf(false)
    }

    Scaffold(modifier = Modifier.padding(16.dp), floatingActionButton = {
        FloatingActionButton(modifier = Modifier
            .offset(x = 16.dp, y = 69.dp)
            .sizeIn(maxHeight = 50.dp, maxWidth = 40.dp), onClick = {
            onEvent(ShoppingListEvent.OpenAddingNewShoppingListItemDialog)

            val randInt = (1..2).random()
            if (randInt==1) shouldShowInterstitialAd = true
            else shouldShowRewardAd = true
        }) {
            Icon(
                imageVector = Icons.Default.Add,
                contentDescription = "Add new shopping list item"
            )
        }
    }, bottomBar = {
        AdMobBannersAd(
            modifier = Modifier
                .height(54.dp)
                .background(Color.White)
                .padding(top = 4.dp),
            contentAlignment = Alignment.TopStart
        )
    }) { padding ->
        if (state.isAddingNewShoppingListItem || state.isEditingShoppingListItem) {
            AddShoppingListItemDialog(state = state, onEvent = onEvent, onShowToast = onShowToast)
        }

        val ctx = LocalContext.current
        if (shouldShowRewardAd) {
            showRewarded(
                context = ctx,
                onShowingAd = {
                    onShowToast("Showing rewarded ad")
                    shouldShowRewardAd = false
                },
                onUserEarnedRewardListener = {
                    onShowToast("Reward granted")
                },
            )
        }
        if (shouldShowInterstitialAd) {
            showInterstitial(
                context = ctx,
                onShowingAd = {
                    onShowToast("Showing interstitial ad")
                    shouldShowInterstitialAd = false
                },
                onLoadedAd = {
                    onShowToast("Loaded interstitial ad")
                }
            )
        }

        Column {
            Row(
                modifier = Modifier.fillMaxWidth(), verticalAlignment = Alignment.CenterVertically
            ) {
                Row(Modifier.padding(end = 8.dp)) {
                    Text(text = "Sort by:", fontSize = 20.sp, fontWeight = FontWeight.Bold)
                }
                Row(Modifier.horizontalScroll(rememberScrollState())) {
                    SortType.entries.forEach { sortType ->
                        Row(
                            modifier = Modifier.clickable {
                                onEvent(
                                    ShoppingListEvent.SetSortType(
                                        sortType
                                    )
                                )
                            }, verticalAlignment = Alignment.CenterVertically
                        ) {
                            RadioButton(selected = state.sortType == sortType,
                                onClick = { onEvent(ShoppingListEvent.SetSortType(sortType)) })
                            Text(text = sortType.name, fontSize = 16.sp)
                        }
                    }
                }
            }

            Divider(
                thickness = 2.dp,
                color = Color.LightGray,
                modifier = Modifier.padding(bottom = 8.dp)
            )

            LazyColumn(
                contentPadding = padding,
                modifier = Modifier
                    .fillMaxSize()
                    .weight(2f),
                verticalArrangement = Arrangement.spacedBy(16.dp)
            ) {
                items(state.shoppingListItems,
                    key = { shoppingListItem -> if (shoppingListItem.id !== null) shoppingListItem.id else 0 }) { shoppingItem ->
                    val dismissState = rememberDismissState(confirmValueChange = {
                        when (it) {
                            DismissValue.DismissedToEnd -> {
                                onEvent(ShoppingListEvent.DeleteShoppingListItem(shoppingItem))
                                onShowToast("Deleted ${shoppingItem.name} shopping list item via swipe gesture")
                                true
                            }

                            DismissValue.DismissedToStart -> {
                                onEvent(
                                    ShoppingListEvent.OpenEditingShoppingListItem(
                                        shoppingItem
                                    )
                                )
                                onShowToast("Opened ${shoppingItem.name} shopping list item edit dialog via swipe gesture")
                                false
                            }

                            else -> {
                                onShowToast("Swipe gesture aborted")
                                false
                            }
                        }
                    })
                    SwipeToDismiss(state = dismissState,
                        background = { SwipeBackground(dismissState = dismissState) },
                        dismissContent = {
                            Row(
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .clip(RoundedCornerShape(10.dp))
                                    .background(Color.LightGray)
                                    .padding(horizontal = 8.dp, vertical = 4.dp)
                            ) {
                                Column(modifier = Modifier.weight(1f)) {
                                    Text(
                                        text = "${shoppingItem.name} - 🧮 ${shoppingItem.count}",
                                        fontSize = 18.sp
                                    )
                                    Text(text = "💶 ${shoppingItem.price}", fontSize = 16.sp)
                                }

                                IconButton(onClick = {
                                    onEvent(ShoppingListEvent.DeleteShoppingListItem(shoppingItem)).also {
                                        onShowToast("Deleted shopping list item ${shoppingItem.name}")
                                    }
                                }) {
                                    Icon(
                                        imageVector = Icons.Default.Delete,
                                        contentDescription = "Delete Shopping List Item"
                                    )
                                }

                                IconButton(onClick = {
                                    onEvent(
                                        ShoppingListEvent.OpenEditingShoppingListItem(
                                            shoppingItem
                                        )
                                    )
                                }) {
                                    Icon(
                                        imageVector = Icons.Default.Edit,
                                        contentDescription = "Edit Shopping List Item"
                                    )
                                }
                            }
                        })
                }
            }
        }
    }
}

@Composable
fun AddShoppingListItemDialog(
    state: ShoppingListState,
    onEvent: (ShoppingListEvent) -> Unit,
    onShowToast: (String) -> Unit?,
    modifier: Modifier = Modifier,
) {
    AlertDialog(modifier = modifier, onDismissRequest = {
        if (state.isAddingNewShoppingListItem) {
            onEvent(ShoppingListEvent.CloseAddingNewShoppingListItemDialog)
        } else if (state.isEditingShoppingListItem) {
            onEvent(ShoppingListEvent.CloseEditingShoppingListItem)
        }
    }, title = { Text(text = "Add new shopping list item") }, text = {
        Column(verticalArrangement = Arrangement.spacedBy(8.dp)) {
            val shoppingListItem = state.newShoppingListItem

            TextField(
                value = shoppingListItem.name,
                onValueChange = { onEvent(ShoppingListEvent.SetName(it)) },
                label = { Text(text = "Item Name") },
                placeholder = { Text(text = "Item Name") },
            )
            TextField(value = shoppingListItem.count,
                label = { Text(text = "Count") },
                placeholder = { Text(text = "Count") },
                onValueChange = {
                    onEvent(ShoppingListEvent.SetCount(count = it))
                },
                keyboardOptions = KeyboardOptions().copy(
                    autoCorrect = false, keyboardType = KeyboardType.NumberPassword
                ),
                leadingIcon = {
                    Text(text = "🧮", fontSize = 24.sp)
                })
            TextField(value = shoppingListItem.price,
                label = { Text(text = "Price") },
                placeholder = { Text(text = "Price") },
                onValueChange = {
                    onEvent(ShoppingListEvent.SetPrice(price = it))
                },
                keyboardOptions = KeyboardOptions().copy(
                    autoCorrect = false, keyboardType = KeyboardType.Decimal
                ),
                leadingIcon = {
                    Text(text = "💶", fontSize = 24.sp)
                })
        }
    }, confirmButton = {
        Button(onClick = {
            if (state.newShoppingListItem.name.length < 3) {
                onShowToast("Item Name is too short!")
                return@Button
            }
            if (state.newShoppingListItem.count.toIntOrNull() === null) {
                onShowToast("Invalid value for Count! Failed to convert value to Int.")
                return@Button
            }
            if (state.newShoppingListItem.price.toFloatOrNull() === null) {
                onShowToast("Invalid value for Price! Failed to convert value to Float.")
                return@Button
            }

            if (state.isAddingNewShoppingListItem) {
                onEvent(ShoppingListEvent.InsertNewShoppingListItemToDb).also { onShowToast("Added new item to shopping list!") }
            } else if (state.isEditingShoppingListItem) {
                onEvent(ShoppingListEvent.UpdateCurrentlyEditingShoppingListItem).also {
                    onShowToast(
                        "Updated shopping list item ${state.currentlyEditingShoppingListItem?.name}!"
                    )
                }
            }
        }) {
            Text(text = if (state.isEditingShoppingListItem) "Update" else "Insert")
        }
    }, dismissButton = {
        Button(onClick = {
            if (state.isAddingNewShoppingListItem) {
                onEvent(ShoppingListEvent.CloseAddingNewShoppingListItemDialog)
            } else if (state.isEditingShoppingListItem) {
                onEvent(ShoppingListEvent.CloseEditingShoppingListItem)
            }
        }) {
            Text(text = "Close")
        }
    })
}

@Composable
@OptIn(ExperimentalMaterial3Api::class)
fun SwipeBackground(dismissState: DismissState) {
    val direction = dismissState.dismissDirection ?: return

    val color by animateColorAsState(
        when (dismissState.targetValue) {
            DismissValue.Default -> Color.Transparent
            DismissValue.DismissedToEnd -> Color.Red
            DismissValue.DismissedToStart -> Color.Green
        }, label = ""
    )
    val alignment = when (direction) {
        DismissDirection.StartToEnd -> Alignment.CenterStart
        DismissDirection.EndToStart -> Alignment.CenterEnd
    }
    val icon = when (direction) {
        DismissDirection.StartToEnd -> Icons.Filled.Delete
        DismissDirection.EndToStart -> Icons.Filled.Edit
    }
    val scale by animateFloatAsState(
        if (dismissState.targetValue == DismissValue.Default) 0.8f else 1.25f, label = ""
    )

    Box(
        Modifier
            .fillMaxSize()
            .background(color)
            .padding(horizontal = 20.dp),
        contentAlignment = alignment
    ) {
        Icon(
            icon, contentDescription = "Delete", modifier = Modifier.scale(scale)
        )
    }
}


@Composable
fun AdMobBannersAd(
    modifier: Modifier = Modifier, contentAlignment: Alignment = Alignment.Center
) {
    Box(modifier = modifier.then(Modifier.fillMaxSize()), contentAlignment = contentAlignment) {
        AndroidView(
            // on below line specifying width for ads.
            factory = { context ->
                // on below line specifying ad view.
                AdView(context).apply {
                    // on below line specifying ad size
                    setAdSize(AdSize.BANNER)
                    // on below line specifying ad unit id
                    // currently added a test ad unit id.
                    adUnitId = "ca-app-pub-3940256099942544/6300978111"
                    // calling load ad to load our ad.
                    loadAd(AdRequest.Builder().build())
                }
            })
    }
}
